/* reaper7 */

#include "driver/i2c.h"
#include "driver/i2c_sht21.h"

bool ICACHE_FLASH_ATTR
SHT21_SoftReset() {
    //Soft reset the SHT21
    i2c_start();
    i2c_writeByte(SHT21_ADDRESS);
    if (!i2c_check_ack()) {
        //os_printf("-%s-%s slave not ack... return \r\n", __FILE__, __func__);
        i2c_stop();
        return(0);
    }
    i2c_writeByte(SOFT_RESET);
    if (!i2c_check_ack()) {
        //os_printf("-%s-%s slave not ack... return \r\n", __FILE__, __func__);
        i2c_stop();
        return(0);
    }
    i2c_stop();
    return(1);
}

bool ICACHE_FLASH_ATTR
SHT21_Init() {
    return(SHT21_SoftReset());
}

/** \note Might be optimized:
Each time a relative humidity measurement is made a temperature measurement is also made for the purposes of
temperature compensation of the relative humidity measurement. If the temperature value is required, it can be
read using command 0xE0; this avoids having to perform a second temperature measurement. The measure
temperature commands 0xE3 and 0xF3 will perform a temperature measurement and return the measurement
value, command 0xE0 does not perform a measurement but returns the temperature value measured during the
relative humidity measurement. The checksum output is not available with the 0xE0 command.
*/
int16_t ICACHE_FLASH_ATTR
SHT21_GetVal(uint8 mode) {
    i2c_start(); //Start i2c
    i2c_writeByte(SHT21_ADDRESS);
    if (!i2c_check_ack()) {
        //os_printf("-%s-%s slave not ack... return \r\n", __FILE__, __func__);
        i2c_stop();
        return(0);
    }

    if (mode==GET_SHT_TEMPERATURE)
        i2c_writeByte(TRIGGER_TEMP_MEASURE_NOHOLD);
    else if (mode==GET_SHT_HUMIDITY)
        i2c_writeByte(TRIGGER_HUMD_MEASURE_NOHOLD); // Measure Relative Humidity, No Hold Master Mode
    else
        return 0;

    if (!i2c_check_ack()) {
        //os_printf("-%s-%s slave not ack... return \r\n", __FILE__, __func__);
        i2c_stop();
        return(0);
    }

    os_delay_us(20);

    i2c_stop();

    os_delay_us(70000);

    uint8 ack = 0;
    while (!ack) {
        i2c_start();
        i2c_writeByte(SHT21_ADDRESS+1);
        ack = i2c_check_ack();
        if (!ack) i2c_stop();
    }
    //os_printf("-%s-%s get ack \r\n", __FILE__, __func__);

    uint8 msb = i2c_readByte();
    i2c_send_ack(1);
    uint8 lsb = i2c_readByte();
    i2c_send_ack(0);
    i2c_stop();

    uint16_t _rv = msb << 8;
    _rv += lsb;
    _rv &= ~0x0003;

    //os_printf("-%s-%s RAW:%d \r\n", __FILE__, __func__,_rv);

    float rv = _rv;

    if (mode==GET_SHT_TEMPERATURE) {
        rv *= 175.72f;
        rv /= 65536;
        rv -= 46.85f;
    } else if (mode==GET_SHT_HUMIDITY) {
        rv *= 125.0f;
        rv /= 65536;
        rv -= 6.0f;
        /*
        Due to normal variations in RH accuracy of the device as described in Table 4, it is possible for the measured value of
        %RH to be slightly less than 0 when the actual RH level is close to or equal to 0. Similarly, the measured value of %RH
        may be slightly greater than 100 when the actual RH level is close to or equal to 100. This is expected behavior, and it is
        acceptable to limit the range of RH results to 0 to 100%RH in the host software by truncating values that are slightly out-
        side of this range.
        */
        if (rv < 0.0f)
            rv = 0.0f;
        else if (rv > 100.0f)
            rv = 100.0f;
    }

    return (int16_t)(rv*10);
}
